/*
 * Copyright (C) 2008 Andrew Olmsted, all rights reserved.
 *
 * Contact: Andrew Olmsted <andrew.olmsted@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <glib-object.h>
#include <stdlib.h>
#include <libosso.h>

#include <hildon/hildon.h>
#include <libhildondesktop/libhildondesktop.h>
#include <stdio.h>
#include <string.h>

#include "statusbar-clock-hours.h"

HD_DEFINE_PLUGIN (StatusbarClockHoursPlugin, statusbar_clock_hours_plugin, STATUSBAR_TYPE_ITEM);

gboolean hours_button_pressed(GtkWidget *widget, gpointer data);
gboolean update_hours(gpointer data);
void launch_clock(GtkMenuItem *item, gpointer data);
void change_24hour_mode(GtkMenuItem *item, gpointer data);
void change_longhorn_mode(GtkMenuItem *item, gpointer data);
void configure_clock(GtkMenuItem *item, gpointer data);
void popup_place(GtkMenu *menu, gint *x, gint *y, gboolean *push_in, gpointer user_data);

static void statusbar_clock_hours_plugin_init(StatusbarClockHoursPlugin *statusbar_plugin)
{
      gchar* filename;
      gboolean fileExists;
      GKeyFile* keyFile;
      GtkWidget* button;
      gchar* fileData;
      FILE* rcFile;
      gsize size;

      statusbar_plugin->hoursLabel=gtk_label_new("");
      button=gtk_button_new();
      gtk_misc_set_padding(GTK_MISC(GTK_BIN(button)->child),0,0);

      gtk_container_add(GTK_CONTAINER(button),statusbar_plugin->hoursLabel);
      gtk_container_add(GTK_CONTAINER(statusbar_plugin), button);
      gtk_misc_set_alignment(GTK_MISC(statusbar_plugin->hoursLabel),1.0,0.5);
      gtk_widget_show_all(statusbar_plugin);

      /* Get whether the user is using 24 hour or AM/PM format */
      keyFile=g_key_file_new();
      filename=g_get_home_dir();
      filename=g_strconcat(filename,"/.statusbar_clock.rc",NULL);
      fileExists=g_key_file_load_from_file(keyFile,filename,G_KEY_FILE_KEEP_COMMENTS,NULL);
      if (!fileExists)
      {
	  g_key_file_set_comment(keyFile,NULL,NULL,"Configuration file for statusbar clock",NULL);
	  g_key_file_set_boolean(keyFile,"initialize","ampm",TRUE);
	  g_key_file_set_boolean(keyFile,"initialize","longhorn",TRUE);
	  g_key_file_set_string(keyFile,"initialize","fonth","Sans Bold 28");
	  g_key_file_set_string(keyFile,"initialize","font colourh","white");
	  fileData=g_key_file_to_data(keyFile,&size,NULL);
	  rcFile=fopen(filename,"w");
	  fputs(fileData,rcFile);
	  fclose(rcFile);
      }
      statusbar_plugin->ampm=g_key_file_get_boolean(keyFile,"initialize","ampm",NULL);
      statusbar_plugin->longhorn=g_key_file_get_boolean(keyFile,"initialize","longhorn",NULL);
      statusbar_plugin->fontDesc=g_key_file_get_string(keyFile,"initialize","fonth",NULL);
      if (!statusbar_plugin->fontDesc)
	  g_key_file_set_string(keyFile,"initialize","fonth","Sans Bold 28");
      statusbar_plugin->fontColour=g_key_file_get_string(keyFile,"initialize","font colourh",NULL);
      if (!statusbar_plugin->fontColour)
	  g_key_file_set_string(keyFile,"initialize","font colourh","white");
      rcFile=fopen(filename,"w");
      fputs(fileData,rcFile);
      fclose(rcFile);
      g_key_file_free(keyFile);

      update_hours(statusbar_plugin);

      statusbar_plugin->eventID=g_timeout_add(1000,update_hours,statusbar_plugin);

      g_signal_connect(button,"clicked",G_CALLBACK(hours_button_pressed),statusbar_plugin);
}

static void statusbar_clock_hours_plugin_finalize(GObject *object)
{
      StatusbarClockHoursPlugin *statusbar_plugin=STATUSBAR_CLOCK_HOURS_PLUGIN(object);
      
      g_source_remove(statusbar_plugin->eventID);
      G_OBJECT_CLASS(g_type_class_peek_parent(G_OBJECT_GET_CLASS(object)))->finalize(object);
      g_free(statusbar_plugin);
}

gboolean update_hours(gpointer data)
{
  StatusbarClockHoursPlugin *infoData=(StatusbarClockHoursPlugin*)data;
  
  time_t curTime;
  struct tm *locTime;
  char timeString[4];
  gchar* filename;
  gboolean fileExists;
  GKeyFile* keyFile;

  keyFile=g_key_file_new();
  filename=g_get_home_dir();
  filename=g_strconcat(filename,"/.statusbar_clock.rc",NULL);
  fileExists=g_key_file_load_from_file(keyFile,filename,G_KEY_FILE_KEEP_COMMENTS,NULL);
  if (fileExists)
  {
      infoData->ampm=g_key_file_get_boolean(keyFile,"initialize","ampm",NULL);
      infoData->longhorn=g_key_file_get_boolean(keyFile,"initialize","longhorn",NULL);
      infoData->fontDesc=g_key_file_get_string(keyFile,"initialize","fonth",NULL);
      infoData->fontColour=g_key_file_get_string(keyFile,"initialize","font colourh",NULL);
  }
  g_key_file_free(keyFile);

  curTime=time(NULL);
  locTime=localtime(&curTime);

  if (infoData->ampm)
  {
	if(infoData->longhorn)
	{
	 strftime(timeString,4,"%l",locTime);
	}
	else
	{
	 strftime(timeString,4,"%l:",locTime);
	}
  }
  else
  {
    strftime(timeString,4,"%H:",locTime);
  }

  gtk_label_set_markup(GTK_LABEL(infoData->hoursLabel),g_strconcat("<span foreground='",infoData->fontColour,"' font-desc='",infoData->fontDesc,"'>",timeString,"</span>",NULL));
  return TRUE;
}

gboolean hours_button_pressed(GtkWidget *widget, gpointer data)
{
  GtkWidget *menu_item;
  time_t curTime;
  struct tm *locTime;
  char* timeString[45];
  StatusbarClockHoursPlugin *infoData=(StatusbarClockHoursPlugin*)data;
  
  curTime=time(NULL);
  locTime=localtime(&curTime);
  strftime(timeString,45,"%A, %B %e, %Y",locTime);

  infoData->hoursMenu = gtk_menu_new();
  menu_item = gtk_menu_item_new_with_label(timeString);
  gtk_widget_set_sensitive(menu_item,FALSE);
  gtk_widget_show_all(menu_item);
  gtk_menu_shell_append(GTK_MENU_SHELL(infoData->hoursMenu), menu_item);

  menu_item = gtk_separator_menu_item_new();
  gtk_widget_show_all(menu_item);
  gtk_menu_shell_append(GTK_MENU_SHELL(infoData->hoursMenu), menu_item);

  menu_item = gtk_menu_item_new_with_label("Open clock");
  g_signal_connect(menu_item, "activate", G_CALLBACK(launch_clock),NULL);
  gtk_widget_show_all(menu_item);
  gtk_menu_shell_append(GTK_MENU_SHELL(infoData->hoursMenu), menu_item);

  menu_item = gtk_check_menu_item_new_with_label("24 hour mode");
  if (!infoData->ampm)
  {
    gtk_check_menu_item_set_active(menu_item,TRUE);
  }
  g_signal_connect(menu_item,"activate",G_CALLBACK(change_24hour_mode),infoData);
  gtk_widget_show_all(menu_item);
  gtk_menu_shell_append(GTK_MENU_SHELL(infoData->hoursMenu), menu_item);

  menu_item = gtk_check_menu_item_new_with_label("Longhorn mode");
  if (infoData->longhorn)
  {
    gtk_check_menu_item_set_active(menu_item,TRUE);
  }
  g_signal_connect(menu_item,"activate",G_CALLBACK(change_longhorn_mode),infoData);
  gtk_widget_show_all(menu_item);
  gtk_menu_shell_append(GTK_MENU_SHELL(infoData->hoursMenu), menu_item);

  menu_item = gtk_menu_item_new_with_label("Configure");
  g_signal_connect(menu_item, "activate", G_CALLBACK(configure_clock),NULL);
  gtk_widget_show_all(menu_item);
  gtk_menu_shell_append(GTK_MENU_SHELL(infoData->hoursMenu), menu_item);

  gtk_menu_popup(GTK_MENU(infoData->hoursMenu), NULL, NULL, popup_place, infoData, 1,
		  gtk_get_current_event_time());

  gtk_widget_show_all(infoData->hoursMenu);
}

void launch_clock(GtkMenuItem *item, gpointer data)
{
	  osso_context_t *context;
        osso_return_t ret;
	  context = osso_initialize("toptest", "0.1", 0,NULL);
        if (context == NULL) {
                printf("could not connect to D-Bus\n");
                return 1;
        }
        ret = osso_rpc_async_run(context,
                                 "com.nokia.osso_worldclock",
                                 "/com/nokia/osso_worldclock",
                                 "com.nokia.osso_worldclock",
                                 "top_application", NULL, NULL,
                                 DBUS_TYPE_STRING,
                                 "this is the top_application parameter",
                                 DBUS_TYPE_INVALID);
        if (ret != OSSO_OK) {
                printf("ERROR!\n");
                return 1;
        }
}

void change_24hour_mode(GtkMenuItem *item, gpointer data)
{
  StatusbarClockHoursPlugin *statusbar_plugin=(StatusbarClockHoursPlugin*)data;
  gchar* filename;
  gboolean fileExists;
  GKeyFile* keyFile;
  
  statusbar_plugin->ampm=!statusbar_plugin->ampm;
  /* Get whether the user is using 24 hour or AM/PM format */
  keyFile=g_key_file_new();
  filename=g_get_home_dir();
  filename=g_strconcat(filename,"/.statusbar_clock.rc",NULL);
  fileExists=g_key_file_load_from_file(keyFile,filename,G_KEY_FILE_KEEP_COMMENTS,NULL);
  if (fileExists)
  {
      gchar* fileData;
      FILE* rcFile;
      gsize size;
      g_key_file_set_comment(keyFile,NULL,NULL,"Configuration file for statusbar clock",NULL);
      g_key_file_set_boolean(keyFile,"initialize","ampm",statusbar_plugin->ampm);
      g_key_file_set_boolean(keyFile,"initialize","longhorn",statusbar_plugin->longhorn);
      fileData=g_key_file_to_data(keyFile,&size,NULL);
      rcFile=fopen(filename,"w");
      fputs(fileData,rcFile);
      fclose(rcFile);
  }
  g_key_file_free(keyFile);
}

void change_longhorn_mode(GtkMenuItem *item, gpointer data)
{
  StatusbarClockHoursPlugin *statusbar_plugin=(StatusbarClockHoursPlugin*)data;
  gchar* filename;
  gboolean fileExists;
  GKeyFile* keyFile;
  
  statusbar_plugin->longhorn=!statusbar_plugin->longhorn;
  /* Get whether the user is using 24 hour or AM/PM format */
  keyFile=g_key_file_new();
  filename=g_get_home_dir();
  filename=g_strconcat(filename,"/.statusbar_clock.rc",NULL);
  fileExists=g_key_file_load_from_file(keyFile,filename,G_KEY_FILE_KEEP_COMMENTS,NULL);
  if (fileExists)
  {
      gchar* fileData;
      FILE* rcFile;
      gsize size;
      g_key_file_set_comment(keyFile,NULL,NULL,"Configuration file for statusbar clock",NULL);
      g_key_file_set_boolean(keyFile,"initialize","ampm",statusbar_plugin->ampm);
      g_key_file_set_boolean(keyFile,"initialize","longhorn",statusbar_plugin->longhorn);
      fileData=g_key_file_to_data(keyFile,&size,NULL);
      rcFile=fopen(filename,"w");
      fputs(fileData,rcFile);
      fclose(rcFile);
  }
  g_key_file_free(keyFile);
}

void configure_clock(GtkMenuItem *item, gpointer data)
{
  StatusbarClockHoursPlugin *statusbar_plugin=(StatusbarClockHoursPlugin*)data;
  GtkWidget *dialog;
  gint response;
  GtkWidget *vbox;
  GtkWidget *captionWidget;
  GtkWidget *fontButton;
  GtkWidget *colourCombo;
  GKeyFile *keyFile;
  gchar* filename;
  gboolean fileExists;
  gchar* colourString;
  gchar* fontString;

  dialog=gtk_dialog_new_with_buttons("Configure Statusbar Clock",NULL,GTK_DIALOG_MODAL,GTK_STOCK_OK,GTK_RESPONSE_ACCEPT,GTK_STOCK_CANCEL,GTK_RESPONSE_REJECT,NULL);

  keyFile=g_key_file_new();
  filename=g_get_home_dir();
  filename=g_strconcat(filename,"/.statusbar_clock.rc",NULL);
  fileExists=g_key_file_load_from_file(keyFile,filename,G_KEY_FILE_KEEP_COMMENTS,NULL);

  vbox=gtk_vbox_new(FALSE,0);

  fontString=g_key_file_get_string(keyFile,"initialize","fonth",NULL);
  if (!fontString)
  {
    fontButton=gtk_font_button_new();
  }
  else
  {
    fontButton=gtk_font_button_new_with_font(fontString);
  }
  captionWidget=hildon_caption_new(NULL,"Choose font",fontButton,NULL,HILDON_CAPTION_MANDATORY);
  gtk_box_pack_start(vbox,captionWidget,FALSE,FALSE,0);

  colourString=g_key_file_get_string(keyFile,"initialize","font colourh",NULL);
  colourCombo=gtk_combo_box_new_text();
  gtk_combo_box_append_text(colourCombo,"white");
  if (g_strcasecmp(colourString,"white")==0)
    gtk_combo_box_set_active(colourCombo,0);
  gtk_combo_box_append_text(colourCombo,"yellow");
  if (g_strcasecmp(colourString,"yellow")==0)
    gtk_combo_box_set_active(colourCombo,1);
  gtk_combo_box_append_text(colourCombo,"red");
  if (g_strcasecmp(colourString,"red")==0)
    gtk_combo_box_set_active(colourCombo,2);
  gtk_combo_box_append_text(colourCombo,"dark red");
  if (g_strcasecmp(colourString,"dark red")==0)
    gtk_combo_box_set_active(colourCombo,3);
  gtk_combo_box_append_text(colourCombo,"orange");
  if (g_strcasecmp(colourString,"orange")==0)
    gtk_combo_box_set_active(colourCombo,4);
  gtk_combo_box_append_text(colourCombo,"dark orange");
  if (g_strcasecmp(colourString,"dark orange")==0)
    gtk_combo_box_set_active(colourCombo,5);
  gtk_combo_box_append_text(colourCombo,"light green");
  if (g_strcasecmp(colourString,"light green")==0)
    gtk_combo_box_set_active(colourCombo,6);
  gtk_combo_box_append_text(colourCombo,"green");
  if (g_strcasecmp(colourString,"green")==0)
    gtk_combo_box_set_active(colourCombo,7);
  gtk_combo_box_append_text(colourCombo,"dark green");
  if (g_strcasecmp(colourString,"dark green")==0)
    gtk_combo_box_set_active(colourCombo,8);
  gtk_combo_box_append_text(colourCombo,"light blue");
  if (g_strcasecmp(colourString,"light blue")==0)
    gtk_combo_box_set_active(colourCombo,9);
  gtk_combo_box_append_text(colourCombo,"blue");
  if (g_strcasecmp(colourString,"blue")==0)
    gtk_combo_box_set_active(colourCombo,10);
  gtk_combo_box_append_text(colourCombo,"dark blue");
  if (g_strcasecmp(colourString,"dark blue")==0)
    gtk_combo_box_set_active(colourCombo,11);
  gtk_combo_box_append_text(colourCombo,"light gray");
  if (g_strcasecmp(colourString,"light gray")==0)
    gtk_combo_box_set_active(colourCombo,12);
  gtk_combo_box_append_text(colourCombo,"gray");
  if (g_strcasecmp(colourString,"gray")==0)
    gtk_combo_box_set_active(colourCombo,13);
  gtk_combo_box_append_text(colourCombo,"dark gray");
  if (g_strcasecmp(colourString,"dark gray")==0)
    gtk_combo_box_set_active(colourCombo,14);
  gtk_combo_box_append_text(colourCombo,"black");
  if (g_strcasecmp(colourString,"black")==0)
    gtk_combo_box_set_active(colourCombo,15);
  captionWidget=hildon_caption_new(NULL,"Choose font colour",colourCombo,NULL,HILDON_CAPTION_MANDATORY);
  gtk_box_pack_start(vbox,captionWidget,FALSE,FALSE,0);

  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),vbox);
  gtk_widget_show_all(dialog);

  response=gtk_dialog_run(dialog);

  if (response==GTK_RESPONSE_ACCEPT)
  {
    gchar* fileData;
    FILE* rcFile;
    gsize size;
    g_key_file_set_string(keyFile,"initialize","fonth",gtk_font_button_get_font_name(GTK_FONT_BUTTON(fontButton)));
    g_key_file_set_string(keyFile,"initialize","font colourh",gtk_combo_box_get_active_text(colourCombo));
    fileData=g_key_file_to_data(keyFile,&size,NULL);
    rcFile=fopen(filename,"w");
    fputs(fileData,rcFile);
    fclose(rcFile);
  }
  gtk_widget_destroy(dialog);
  g_key_file_free(keyFile);
}

void popup_place(GtkMenu *menu, gint *x, gint *y, gboolean *push_in, gpointer user_data)
{
      StatusbarClockHoursPlugin *statusbar_plugin=(StatusbarClockHoursPlugin*)user_data;
	GtkWidget *btn=statusbar_plugin->hoursLabel;

	gdk_window_get_position(btn->window, x, y);

	*y += btn->allocation.y + btn->allocation.height;
	*x += btn->allocation.x;
}

static void statusbar_clock_hours_plugin_class_init(StatusbarClockHoursPluginClass *klass)
{
  GObjectClass *object_class=G_OBJECT_CLASS(klass);
  object_class->finalize=statusbar_clock_hours_plugin_finalize;
}
